/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.admin.autostart.view {

    import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.events.TimerEvent;
import flash.utils.Timer;

import mx.controls.Button;

import mx.controls.ProgressBar;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jonas.admin.autostart.event.*;
    import org.ow2.jonas.admin.autostart.model.*;
    import org.ow2.jonas.admin.autostart.vo.*;

    import mx.controls.TextArea;
    import mx.containers.VBox;
    import mx.collections.ArrayCollection;
    import mx.controls.Alert;
    import flash.net.FileReference;

    public class MainViewClass extends VBox {

        public function MainViewClass() {
        }

        [Bindable]
        public var model : BuilderModelLocator =
            BuilderModelLocator.getInstance();

        private var buildProcessCompleteTimer:Timer = new Timer(5000);
        private var getOutputContentTimer:Timer = new Timer(10);
        private var saveStarterTimer:Timer = new Timer(5000);
        private var setStarterPathTimer : Timer = new Timer(10000);
        private var btnBackButton : Button = new Button();

        /**
         * Initialize JOnAS Versions list.
         */
        public function init() : void {
            var builderVo : BuilderVO = new BuilderVO();
            var event : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_JONAS_VERSION + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(event);
            var eventFolders : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_JONAS_FOLDERS + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(eventFolders);
        }

        /**
         * Reload the available applications list.
         */
        public function reloadAvailableAppliList() : void {
            var builderVo : BuilderVO = new BuilderVO();
            var eventAvailableAppli : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_AVAILABLE_APPLI + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(
                eventAvailableAppli);
        }

        /**
         * Launch the build process.
         * @param backButton
         */
        public function startBuildProcess(backButton : Button) : void {
            btnBackButton = backButton;
            btnBackButton.enabled = false;
            model.outputText = "";
            model.progressBar.indeterminate = true;

            var tmpName : String = model.jonasStarterName.slice(-4,
                model.jonasStarterName.length);
            if (tmpName != ".jar") {
                    model.jonasStarterName = model.jonasStarterName + ".jar";
            }
            model.builderVO.jonasStarterName = model.jonasStarterName;
            model.builderVO.applications = model.uploadedApplication;
            model.builderVO.jonasVersion = model.jonasSelectedVersion;
            model.builderVO.jonasType = model.jonasSelectedType;
            model.builderVO.getJonasFrom = model.getJonasFrom;
            model.builderVO.jonasZipFile = model.jonasZipFile;
            model.builderVO.parameter = "###";
            model.builderVO.applyConfiguration = model.applyConfiguration;
            model.builderVO.useFullStarter = model.useFullStarter;
            model.builderVO.includeJonasClient = model.includeJonasClient;

            /**
             * JOnAS server configuration.
             */
            model.builderVO.configuration.webContainerVo.httpPort
                = model.httpPort;
            model.builderVO.configuration.webContainerVo.redirectPort
                = model.redirectPort;
            model.builderVO.configuration.webContainerVo.useSsl
                = model.useSsl;
            model.builderVO.configuration.webContainerVo.activateJvmRoute
                = model.activateJvmRoute;

            model.builderVO.configuration.protocolsVo.protocolList
                = model.protocolList;
            model.builderVO.configuration.protocolsVo.jrmpPort
                = model.jrmpPort;
            model.builderVO.configuration.protocolsVo.irmiPort = model.irmiPort;
            model.builderVO.configuration.protocolsVo.iiopPort = model.iiopPort;
            model.builderVO.configuration.protocolsVo.lmiPort = model.lmiPort;

            model.builderVO.configuration.servicesVo.servicesList
                = model.servicesList;

            model.builderVO.configuration.jmsVo.jmsPort = model.jmsPort;
            model.builderVO.configuration.jmsVo.queueList = model.queueList;
            model.builderVO.configuration.jmsVo.topicList = model.topicList;

            model.builderVO.configuration.hsqlDbVo.dbPort = model.dbPort;
            model.builderVO.configuration.hsqlDbVo.dbName = model.dbName;
            model.builderVO.configuration.hsqlDbVo.dbUser = model.dbUser;
            model.builderVO.configuration.hsqlDbVo.dbUserpasswd
                = model.dbUserpasswd;

            /*var object : Object = new Object();
            object.jonasStarterName = model.jonasStarterName;
            object.applications = model.uploadedApplication;
            object.jonasVersion = model.jonasSelectedVersion;
            object.jonasType = model.jonasSelectedType;
            object.jonasZipFile = model.jonasZipFile;
            object.parameter = "model.parameter";
            object.applyConfiguration = model.applyConfiguration;
            object.getJonasFrom = model.getJonasFrom;
            object.useFullStarter = model.useFullStarter;
            object.includeJonasClient = model.includeJonasClient;
            //model.builderVO = new BuilderVO();
            model.builderVO.init(object);*/
            var event : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_START_BUILD_PROCESS + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(event);

            buildProcessCompleteTimer.addEventListener(TimerEvent.TIMER,
                addStarter);
            buildProcessCompleteTimer.start();

            getOutputContentTimer.addEventListener(TimerEvent.TIMER,
                getOutputContent);
            getOutputContentTimer.start();/**/
        }

        /**
         * Add a new starter to the list.
         * @param event
         */
        private function addStarter(event:TimerEvent) : void {
            //Alert.show(model.builderVO.getJonasFrom + " :::");
            if (model.buildStatus != "Error") {
                var object : Object = new Object();
                var starterVo : StarterVO = new StarterVO();
                object.jonasStarterName = model.jonasStarterName;
                object.jonasStarterDescription =
                    model.jonasStarterDescription;
                object.jonasStarterPath = "...";
                var date : Date = new Date();
                var month:uint = date.getMonth() + 1;
                var stringDate : String =
                        "" + date.getFullYear() + "-" + month + "-"
                        + date.getDate() + ":" +
                        + date.getHours() + ":" + date.getMinutes()
                        + ":" + date.getSeconds();
                object.jonasStarterCreationDate = stringDate.toString();
                object.applications = model.uploadedApplication;
                starterVo.init(object);
                starterVo.jonasStarterName = model.jonasStarterName;
                starterVo.jonasStarterDescription =
                    model.jonasStarterDescription;
                starterVo.jonasStarterPath = "...";
                starterVo.jonasStarterCreationDate = stringDate.toString();
                starterVo.applications = model.uploadedApplication;

                if (!model.jonasStarterList.contains(
                        starterVo.jonasStarterName)) {
                    model.builderVO.starters.addItem(starterVo);
                    model.jonasStarterList.addItem(starterVo.jonasStarterName);

                    model.uploadedApplication = new ArrayCollection();
                    var id : String = "_"
                        + date.getDate() + month + date.getFullYear()
                        + date.getHours() + date.getMinutes()
                        + date.getSeconds();
                    model.jonasStarterName = "jonas-starter" + id + ".jar";
                    model.jonasStarterDescription = "";
                }

                var eventSaveStarter : BuilderEvent =
                    new BuilderEvent(BuilderEvent.EVENT_SAVE_STARTER_INFO_IN_FILE + model.componentID);
                CairngormEventDispatcher.getInstance().dispatchEvent(
                    eventSaveStarter);

                btnBackButton.enabled = true;
                model.buildStatus = "Error";

                model.progressBar.minimum = 100;
                model.progressBar.indeterminate = false;
                buildProcessCompleteTimer.stop();
                getOutputContentTimer.stop();
            }
        }

        /**
         * Add a new starter to the list.
         * @param event
         */
        private function getOutputContent(event:TimerEvent) : void {
            var builderVO:BuilderVO = new BuilderVO();
            var getOutputevent : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_GET_OUTPUT_CONTENT + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(
                getOutputevent);
        }

        public function downloadJonasZipFileEvent(url:String) : void {
            var builderVo : BuilderVO = new BuilderVO();
            builderVo.parameter = url;
            var eventDownloadFile : BuilderEvent =
            new BuilderEvent(BuilderEvent.EVENT_DOWNLOAD_ZIP_FILE + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(
                eventDownloadFile);
        }

        public function saveStarterEvent(event:TimerEvent) : void {
            //Save save in an xml file.
            var eventSaveStarter : BuilderEvent =
                new BuilderEvent(BuilderEvent.EVENT_SAVE_STARTER_INFO_IN_FILE + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(
                eventSaveStarter);
        }

        public function isValidUrl(url : String) : void {
            var builderVo : BuilderVO = new BuilderVO();
            builderVo.parameter = url;
            var eventDownloadFile : BuilderEvent =
            new BuilderEvent(BuilderEvent.EVENT_IS_VALID_URL + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(
                eventDownloadFile);
        }

        public function stopAllTimer() : void {
            saveStarterTimer.stop();
            setStarterPathTimer.stop();
            buildProcessCompleteTimer.stop();
            getOutputContentTimer.stop();
            var event : BuilderEvent =
            new BuilderEvent(BuilderEvent.EVENT_STOP_BUILD_PROCESS + model.componentID);
            CairngormEventDispatcher.getInstance().dispatchEvent(event);
        }
    }
}
